use self::random::Xoshiro256SS;
pub use self::template::*;
use std::collections::VecDeque;
use std::env;
use std::fmt::{Display, Formatter};
use std::io::{stdin, stdout, BufWriter, Write};

#[allow(dead_code)]
fn gen_king_moves() -> [(i8, i8); 8] {
    let mut ans: [(i8, i8); 8] = Default::default();
    let mut pnt = 0;

    for dx in -1..=1 {
        for dy in -1..=1 {
            if dx != 0 || dy != 0 {
                ans[pnt] = (dx, dy);
                pnt += 1;
            }
        }
    }

    ans
}

#[allow(dead_code)]
fn gen_camel_moves() -> [(i8, i8); 8] {
    let mut ans: [(i8, i8); 8] = Default::default();
    let mut pnt = 0;

    for (ax, ay) in [(1, 3), (3, 1)] {
        for dx in [-ax, ax] {
            for dy in [-ay, ay] {
                ans[pnt] = (dx, dy);
                pnt += 1;
            }
        }
    }

    ans
}

const KING_MOVES: [(i8, i8); 8] = [
    (-1, -1),
    (-1, 0),
    (-1, 1),
    (0, -1),
    (0, 1),
    (1, -1),
    (1, 0),
    (1, 1),
];

const CAMEL_MOVES: [(i8, i8); 8] = [
    (-1, -3),
    (-1, 3),
    (1, -3),
    (1, 3),
    (-3, -1),
    (-3, 1),
    (3, -1),
    (3, 1),
];

const WAZIR_MOVES: [(i8, i8); 4] = [(1, 0), (-1, 0), (0, 1), (0, -1)];

#[derive(Debug, Default, Copy, Clone, Ord, PartialOrd, Eq, PartialEq, Hash)]
struct Position {
    black_move: bool,
    white_king: (u8, u8),
    black_king: (u8, u8),
    camel: (u8, u8),
    wazir: (u8, u8),
}

impl Display for Position {
    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
        fn to_algebraic((x, y): (u8, u8)) -> String {
            [('a' as u8 + x) as char, ('1' as u8 + y) as char]
                .iter()
                .collect()
        }

        write!(
            f,
            "{} {} {} {} {}",
            to_algebraic(self.white_king),
            to_algebraic(self.black_king),
            to_algebraic(self.camel),
            to_algebraic(self.wazir),
            if self.black_move { 'b' } else { 'w' }
        )
    }
}

impl Position {
    pub fn to_array(self) -> [u8; 9] {
        [
            self.black_move as u8,
            self.white_king.0,
            self.white_king.1,
            self.black_king.0,
            self.black_king.1,
            self.camel.0,
            self.camel.1,
            self.wazir.0,
            self.wazir.1,
        ]
    }

    pub fn to_usize(mut self) -> usize {
        self = self.to_canonical();

        debug_assert!(self.white_king.0 < 4 && self.white_king.1 < 4);

        let mut value =
            ((self.black_move as u8 * 4 + self.white_king.0) * 4 + self.white_king.1) as usize;

        for &x in &self.to_array()[3..] {
            value *= 8;
            value += x as usize;
        }

        assert!(value < 1 << (6 * 4 - 1));

        value
    }

    pub fn from_array(arr: [u8; 9]) -> Self {
        Self {
            black_move: arr[0] > 0,
            white_king: (arr[1], arr[2]),
            black_king: (arr[3], arr[4]),
            camel: (arr[5], arr[6]),
            wazir: (arr[7], arr[8]),
        }
    }

    pub fn from_usize(mut value: usize) -> Self {
        let mut arr = [0; 9];

        for i in 0..arr.len() - 3 {
            arr[i] = (value % 8) as u8;
            value /= 8;
        }
        for i in arr.len() - 3..arr.len() - 1 {
            arr[i] = (value % 4) as u8;
            value /= 4;
        }
        debug_assert!(value < 2);
        arr[8] = value as u8;

        arr.reverse();
        Self::from_array(arr)
    }

    pub fn reflect(self) -> Self {
        Self {
            black_move: self.black_move,
            white_king: (7 - self.white_king.0, self.white_king.1),
            black_king: (7 - self.black_king.0, self.black_king.1),
            camel: (7 - self.camel.0, self.camel.1),
            wazir: (7 - self.wazir.0, self.wazir.1),
        }
    }

    pub fn transpone(self) -> Self {
        Self {
            black_move: self.black_move,
            white_king: (self.white_king.1, self.white_king.0),
            black_king: (self.black_king.1, self.black_king.0),
            camel: (self.camel.1, self.camel.0),
            wazir: (self.wazir.1, self.wazir.0),
        }
    }

    pub fn rotate(self) -> Self {
        self.transpone().reflect()
    }

    pub fn to_canonical(self) -> Self {
        if self.white_king.0 < 4 && self.white_king.1 < 4 {
            self
        } else {
            self.rotate().to_canonical()
        }
    }

    pub fn is_valid(self) -> bool {
        if !self.to_array().iter().all(|&x| x < 8) {
            return false;
        }

        let mut white = [self.white_king, self.camel, self.wazir];
        white.sort_unstable();

        if white[0] == white[1] || white[1] == white[2] {
            return false;
        }

        if self
            .white_king
            .0
            .abs_diff(self.black_king.0)
            .max(self.white_king.1.abs_diff(self.black_king.1))
            < 2
        {
            return false;
        }

        if self.black_move && (self.black_king == self.camel || self.black_king == self.wazir) {
            return false;
        }

        self.black_move || !self.is_check()
    }

    pub fn is_check(self) -> bool {
        if self.black_king.0.abs_diff(self.wazir.0) + self.black_king.1.abs_diff(self.wazir.1) == 1
        {
            return true;
        }
        let dx = self.black_king.0.abs_diff(self.camel.0);
        let dy = self.black_king.1.abs_diff(self.camel.1);

        [dx, dy] == [1, 3] || [dx, dy] == [3, 1]
    }

    pub fn is_insufficient_material(self) -> bool {
        self.black_king == self.wazir || self.black_king == self.camel
    }

    pub fn is_checkmate(self) -> bool {
        self.black_move && self.is_check() && self.gen_moves().is_empty()
    }

    pub fn is_stalemate(self) -> bool {
        self.black_move && !self.is_check() && self.gen_moves().is_empty()
    }

    pub fn move_white_king(mut self, shift: (i8, i8)) -> Self {
        // debug_assert!(!self.black_move);

        self.black_move = !self.black_move;
        self.white_king = (
            self.white_king.0.wrapping_add_signed(shift.0),
            self.white_king.1.wrapping_add_signed(shift.1),
        );

        self
    }

    pub fn move_black_king(mut self, shift: (i8, i8)) -> Self {
        // debug_assert!(self.black_move);

        self.black_move = !self.black_move;
        self.black_king = (
            self.black_king.0.wrapping_add_signed(shift.0),
            self.black_king.1.wrapping_add_signed(shift.1),
        );

        self
    }

    pub fn move_camel(mut self, shift: (i8, i8)) -> Self {
        // debug_assert!(!self.black_move);

        self.black_move = !self.black_move;
        self.camel = (
            self.camel.0.wrapping_add_signed(shift.0),
            self.camel.1.wrapping_add_signed(shift.1),
        );

        self
    }

    pub fn move_wazir(mut self, shift: (i8, i8)) -> Self {
        // debug_assert!(!self.black_move);

        self.black_move = !self.black_move;
        self.wazir = (
            self.wazir.0.wrapping_add_signed(shift.0),
            self.wazir.1.wrapping_add_signed(shift.1),
        );

        self
    }

    pub fn gen_moves(self) -> Vec<Position> {
        let mut moves = Vec::new();

        if self.black_move {
            for m in KING_MOVES {
                let s = self.move_black_king(m);

                if s.is_valid() && !s.is_check() {
                    moves.push(s.to_canonical());
                }
            }
        } else {
            for m in KING_MOVES {
                let s = self.move_white_king(m);

                if s.is_valid() {
                    moves.push(s.to_canonical());
                }
            }
            for m in CAMEL_MOVES {
                let s = self.move_camel(m);

                if s.is_valid() {
                    moves.push(s.to_canonical());
                }
            }
            for m in WAZIR_MOVES {
                let s = self.move_wazir(m);

                if s.is_valid() {
                    moves.push(s.to_canonical());
                }
            }
        }

        moves.sort_unstable();
        moves.dedup();

        moves
    }

    pub fn gen_anti_moves(self) -> Vec<Position> {
        let mut moves = Vec::new();

        if !self.black_move {
            for m in KING_MOVES {
                let s = self.move_black_king(m);

                if s.is_valid() {
                    moves.push(s.to_canonical());
                }
            }
        } else {
            for m in KING_MOVES {
                let s = self.move_white_king(m);

                if s.is_valid() && !s.is_check() {
                    moves.push(s.to_canonical());
                }
            }
            for m in CAMEL_MOVES {
                let s = self.move_camel(m);

                if s.is_valid() && !s.is_check() {
                    moves.push(s.to_canonical());
                }
            }
            for m in WAZIR_MOVES {
                let s = self.move_wazir(m);

                if s.is_valid() && !s.is_check() {
                    moves.push(s.to_canonical());
                }
            }
        }

        moves.sort_unstable();
        moves.dedup();

        moves
    }
}

impl From<usize> for Position {
    fn from(value: usize) -> Self {
        Self::from_usize(value)
    }
}

impl From<Position> for usize {
    fn from(value: Position) -> usize {
        value.to_usize()
    }
}

const UNKNOWN: u8 = u8::MAX;
const INVALID: u8 = UNKNOWN - 1;
const DRAW: u8 = INVALID - 1;

#[allow(dead_code)]
fn retro() -> Vec<u8> {
    let mut dp = vec![u8::MAX; 1 << (6 * 4 - 1)];
    let mut deg = vec![1; dp.len()];

    let mut q = VecDeque::new();

    for j in 0..dp.len() {
        let p = Position::from(j);

        if p.is_valid() {
            if p.is_stalemate() || p.is_insufficient_material() {
                q.push_back((j, DRAW));
            } else if p.is_checkmate() {
                q.push_back((j, 0));
            } else {
                deg[j] = p.gen_moves().len() as u8;
            }
        } else {
            dp[j] = INVALID;
        }
    }

    while let Some((j, res)) = q.pop_front() {
        assert_ne!(res, UNKNOWN);

        if dp[j] != UNKNOWN {
            continue;
        }
        dp[j] = res;

        let p = Position::from(j);

        if p.black_move {
            if res == DRAW {
                for prev in p
                    .gen_anti_moves()
                    .iter()
                    .map(|w| w.to_usize())
                    .filter(|&j| dp[j] == UNKNOWN)
                {
                    deg[prev] -= 1;

                    if deg[prev] == 0 {
                        q.push_back((prev, DRAW));
                    }
                }
            } else {
                for prev in p
                    .gen_anti_moves()
                    .iter()
                    .map(|w| w.to_usize())
                    .filter(|&j| dp[j] == UNKNOWN)
                {
                    deg[prev] -= 1;

                    q.push_back((prev, res + 1));
                }
            }
        } else {
            if res == DRAW {
                for prev in p
                    .gen_anti_moves()
                    .iter()
                    .map(|w| w.to_usize())
                    .filter(|&j| dp[j] == UNKNOWN)
                {
                    deg[prev] -= 1;

                    q.push_back((prev, DRAW));
                }
            } else {
                for prev in p
                    .gen_anti_moves()
                    .iter()
                    .map(|w| w.to_usize())
                    .filter(|&j| dp[j] == UNKNOWN)
                {
                    deg[prev] -= 1;

                    if deg[prev] == 0 {
                        q.push_back((prev, res));
                    }
                }
            }
        }
    }

    for j in &mut dp {
        if *j == UNKNOWN {
            *j = DRAW;
        }
    }

    dp
}

pub fn solve(_scan: &mut impl Scanner, cout: &mut impl Write) {
    let ans = retro();
    let args: Vec<_> = env::args().collect();
    let val = args[1].parse().unwrap();
    let cnt: usize = args[2].parse().unwrap();

    let mut q: Vec<_> = (0..(1 << (6 * 4 - 1)))
        .filter(|j| {
            let p = Position::from(*j);
            p.is_valid()
                && !p.is_checkmate()
                && !p.is_insufficient_material()
                && !p.is_stalemate()
                && ![INVALID, DRAW, UNKNOWN].contains(&ans[*j])
                && ans[*j] > val
        })
        .collect();
    q.extend((0..(1 << (6 * 4 - 1))).filter(|j| {
        let p = Position::from(*j);
        p.is_valid()
            && !p.is_checkmate()
            && !p.is_insufficient_material()
            && !p.is_stalemate()
            && ans[*j] == val
    }));

    if q.len() > cnt / 4 {
        q.truncate(cnt / 4);
    }

    let mut rng = Xoshiro256SS::new(736);
    rng.shuffle(q.as_mut_slice());

    writeln!(cout, "{}", q.len() * 4).unwrap();

    for j in q {
        let mut p = Position::from(j);

        for _ in 0..4 {
            writeln!(cout, "{}", p).unwrap();
            p = p.rotate();
        }
    }

    cout.flush().unwrap();
}

#[allow(dead_code)]
fn main() {
    solve(
        &mut IOScanner::new(stdin().lock()),
        &mut BufWriter::new(stdout().lock()),
    );
}

pub mod template {
    use std::io::BufRead;
    use std::str::FromStr;

    pub trait Scanner {
        fn checked_next_line(&mut self) -> Option<String>;

        fn checked_next<T: FromStr>(&mut self) -> Option<T>;

        #[allow(dead_code)]
        fn next<T: FromStr>(&mut self) -> T;

        fn seek(&mut self) -> bool;
    }

    pub struct IOScanner<ReadType: BufRead> {
        read: ReadType,
        buffer: Vec<String>,
    }

    impl<ReadType: BufRead> Scanner for IOScanner<ReadType> {
        fn checked_next_line(&mut self) -> Option<String> {
            self.buffer.clear();
            let mut input = String::new();
            if self.read.read_line(&mut input).expect("Failed read") > 0 {
                Some(input)
            } else {
                None
            }
        }

        fn checked_next<T: FromStr>(&mut self) -> Option<T> {
            loop {
                if let Some(token) = self.buffer.pop() {
                    return Some(token.parse().ok().expect("Failed parse"));
                }
                if let Some(input) = self.checked_next_line() {
                    self.buffer = input.split_whitespace().rev().map(String::from).collect();
                } else {
                    return None;
                }
            }
        }

        fn next<T: FromStr>(&mut self) -> T {
            self.checked_next().expect("Read after EOF")
        }

        fn seek(&mut self) -> bool {
            let mut input = String::new();

            while let Ok(cnt) = self.read.read_line(&mut input) {
                if cnt == 0 {
                    break;
                }
                let line = input.trim();
                if !line.is_empty() && line.matches("=").count() == line.len() {
                    return true;
                }
                input.clear();
            }

            false
        }
    }

    impl<ReadType: BufRead> IOScanner<ReadType> {
        pub fn new(read: ReadType) -> IOScanner<ReadType> {
            IOScanner {
                read,
                buffer: Vec::<String>::default(),
            }
        }
    }

    pub(crate) mod random {
        use std::collections::Bound;
        use std::ops::RangeBounds;

        #[derive(Debug, Clone, Ord, PartialOrd, Eq, PartialEq, Hash)]
        pub struct Xoshiro256SS {
            state: [u64; 4],
        }

        #[allow(dead_code)]
        impl Xoshiro256SS {
            fn splitmix64(state: &mut u64) -> u64 {
                *state = state.wrapping_add(0x9E3779B97f4A7C15u64);
                let mut result = state.clone();
                result = (result ^ (result >> 30)).wrapping_mul(0xBF58476D1CE4E5B9);
                result = (result ^ (result >> 27)).wrapping_mul(0x94D049BB133111EB);
                return result ^ (result >> 31);
            }

            pub fn new(mut seed: u64) -> Self {
                let mut state = [0; 4];

                for it in &mut state {
                    *it = Self::splitmix64(&mut seed);
                }

                Self { state }
            }

            pub fn gen(&mut self) -> u64 {
                let s = &mut self.state;
                let result = s[1].wrapping_mul(5).rotate_left(7).wrapping_mul(9);
                let t = s[1] << 17;

                s[2] ^= s[0];
                s[3] ^= s[1];
                s[1] ^= s[2];
                s[0] ^= s[3];

                s[2] ^= t;
                s[3] = s[3].rotate_left(45);

                return result;
            }

            pub fn gen_modulo(&mut self, modulo: u64) -> u64 {
                self.gen() % modulo
            }

            pub fn gen_range(&mut self, range: impl RangeBounds<u64>) -> u64 {
                if range.start_bound() == Bound::Unbounded && range.end_bound() == Bound::Unbounded
                {
                    return self.gen();
                }

                let start = match range.start_bound() {
                    Bound::Included(x) => *x,
                    Bound::Excluded(x) => x + 1,
                    Bound::Unbounded => 0,
                };

                let len = match range.end_bound() {
                    Bound::Included(x) => x.wrapping_add(1),
                    Bound::Excluded(x) => *x,
                    Bound::Unbounded => 0,
                }
                .wrapping_sub(start);

                self.gen_modulo(len) + start
            }

            pub fn shuffle<T>(&mut self, arr: &mut [T]) {
                for i in 1..arr.len() {
                    arr.swap(i, self.gen_modulo((i + 1) as u64) as usize);
                }
            }
        }
    }
}
