use std::convert::From;
use std::io;
use std::io::prelude::*;
use std::str::FromStr;

#[cfg(dprintln)]
#[allow(unused_macros)]
macro_rules! dprintln { ($( $args:expr ),*) => { eprintln!( $( $args ),* ); } }

#[cfg(not(dprintln))]
#[allow(unused_macros)]
macro_rules! dprintln {
  ($( $args:expr ),*) => {};
}

struct Bits64(u64);
impl Iterator for Bits64 {
  type Item = usize;
  fn next(&mut self) -> Option<Self::Item> {
    if self.0 == 0 {
      None
    } else {
      let i = self.0.trailing_zeros() as usize;
      self.0 &= self.0 - 1;
      Some(i)
    }
  }
}

fn dd(v: (i32, i32)) -> i32 {
  v.0 * v.0 + v.1 * v.1
}

#[derive(Clone, Copy, PartialEq, Debug)]
struct Cell(u32);
impl Cell {
  fn col(&self) -> u32 {
    self.0 & 7
  }
  fn row(&self) -> u32 {
    self.0 >> 3
  }
  fn mirror_col(&mut self) {
    let col = self.col();
    self.0 = (self.0 & 0o70) + (7 - col);
  }
  fn mirror_row(&mut self) {
    let row = self.row();
    self.0 = (self.0 & 7) + ((7 - row) << 3);
  }
  fn mirror_diag(&mut self) {
    let col = self.col();
    let row = self.row();
    self.0 = (col << 3) + row;
  }
  fn unpack(&self) -> (i32, i32) {
    (self.row() as i32, self.col() as i32)
  }
  fn delta_vector(&self, other: &Self) -> (i32, i32) {
    let (y1, x1) = self.unpack();
    let (y2, x2) = other.unpack();
    (y2 - y1, x2 - x1)
  }
  fn attacked_by_king(&self, king: &Self) -> bool {
    let (vy, vx) = self.delta_vector(king);
    vy.abs().max(vx.abs()) <= 1
  }
  fn attacked_by_knight(&self, knight: &Self) -> bool {
    dd(self.delta_vector(knight)) == 10
  }
  fn attacked_by_rook(&self, rook: &Self) -> bool {
    dd(self.delta_vector(rook)) == 1
  }
}

#[derive(Debug, PartialEq, Eq)]
struct ParseCellError(String);

impl FromStr for Cell {
  type Err = ParseCellError;
  fn from_str(s: &str) -> Result<Self, Self::Err> {
    dprintln!("{}", s);
    if s.len() != 2 {
      return Err(ParseCellError(String::from("illegal cell str len")));
    }
    let mut it = s.chars();
    let c = it.next().unwrap();
    if c < 'a' || c > 'h' {
      return Err(ParseCellError(String::from("illegal col character")));
    }
    let col = c as u8 - b'a';
    let c = it.next().unwrap();
    if c < '1' || c > '8' {
      return Err(ParseCellError(String::from("illegal row character")));
    }
    let row = c as u8 - b'1';
    Ok(Cell(8 * row as u32 + col as u32))
  }
}

const ILLEGAL: i8 = 127;
const DRAW: i8 = 126;
const UNKNOWN: i8 = 125;

#[derive(Clone, Debug, PartialEq)]
struct Pos {
  wk: Cell,
  bk: Cell,
  wn: Cell, //верблюд
  wr: Cell, //визирь
  side: i8, /* 1 - white, -1 - black */
}

struct Bitboard([u64; 64]);
impl Bitboard {
  fn new<F: Fn(Cell, Cell) -> bool>(f: F) -> Self {
    let mut a: [u64; 64] = [0; 64];
    for i in 0..64 {
      let ci = Cell(i);
      a[i as usize] = (0..64)
        .filter_map(|j| {
          if i != j && f(ci, Cell(j)) {
            Some(1u64 << j)
          } else {
            None
          }
        })
        .sum::<u64>();
    }
    Self(a)
  }
}

struct Bitboards {
  king: Bitboard,
  knight: Bitboard,
  rook: Bitboard,
}

impl Default for Bitboards {
  fn default() -> Self {
    Bitboards {
      king: Bitboard::new(|i, j| i.attacked_by_king(&j)),
      knight: Bitboard::new(|i, j| i.attacked_by_knight(&j)),
      rook: Bitboard::new(|i, j| i.attacked_by_rook(&j)),
    }
  }
}

const NO_TO_CELL: [u8; 10] = [0, 1, 2, 3, 9, 10, 11, 18, 19, 27];
const CELL_TO_NO: [u8; 28] = [
  0, 1, 2, 3, 0xff, 0xff, 0xff, 0xff, 0xff, 4, 5, 6, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 7, 8,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 9,
];

impl From<u32> for Pos {
  fn from(x: u32) -> Pos {
    let bk = Cell(x & 63);
    let wn = Cell((x >> 6) & 63);
    let wr = Cell((x >> 12) & 63);
    let side = if (x & (1 << 18)) != 0 { -1 } else { 1 };
    let wk = Cell(NO_TO_CELL[(x >> 19) as usize] as u32);
    Pos {
      wk,
      bk,
      wn,
      wr,
      side,
    }
  }
}

impl From<Pos> for usize {
  fn from(p: Pos) -> usize {
    let mut p = p;
    if p.wk.col() >= 4 {
      p.mirror_col();
    }
    if p.wk.row() >= 4 {
      p.mirror_row();
    }
    if p.wk.row() > p.wk.col() {
      p.mirror_diag();
    }
    let t = CELL_TO_NO[p.wk.0 as usize] as u32;
    debug_assert!(t < 10);
    let r = (((1 - p.side) as u32) << 17) + (p.wr.0 << 12) + (p.wn.0 << 6) + p.bk.0 + (t << 19);
    r as _
  }
}

impl Pos {
  fn white_attacked_cells(&self, b: &Bitboards) -> u64 {
    b.king.0[self.wk.0 as usize] | b.rook.0[self.wr.0 as usize] | b.knight.0[self.wn.0 as usize]
  }
  fn white_pieces(&self) -> u64 {
    (1u64 << self.wk.0) | (1u64 << self.wn.0) | (1u64 << self.wr.0)
  }
  fn black_moves(&mut self, b: &Bitboards) -> (Vec<u32>, bool) {
    debug_assert_eq!(self.side, -1);
    let wa = self.white_attacked_cells(b);
    let u = !wa & b.king.0[self.bk.0 as usize];
    let mut r = Vec::with_capacity(u.count_ones() as usize);
    let bk = self.bk;
    self.side *= -1;
    for w in Bits64(u) {
      self.bk = Cell(w as u32);
      debug_assert!(!self.check());
      r.push(usize::from(self.clone()) as u32);
    }
    self.bk = bk;
    self.side *= -1;
    (r, wa & (1u64 << bk.0) != 0)
  }
  fn white_moves(&mut self, b: &Bitboards) -> Vec<u32> {
    debug_assert_eq!(self.side, 1);
    let wp = self.white_pieces();
    self.side *= -1;
    let mut r = Vec::with_capacity(20);
    //king moves
    let wk = self.wk;
    for w in Bits64(b.king.0[wk.0 as usize] & !wp & !b.king.0[self.bk.0 as usize]) {
      self.wk = Cell(w as u32);
      if !self.draw(b) {
        r.push(usize::from(self.clone()) as u32);
      }
    }
    self.wk = wk;
    let wr = self.wr;
    for w in Bits64(b.rook.0[wr.0 as usize] & !wp) {
      self.wr = Cell(w as u32);
      if !self.draw(b) {
        r.push(usize::from(self.clone()) as u32);
      }
    }
    self.wr = wr;
    let wn = self.wn;
    for w in Bits64(b.knight.0[wn.0 as usize] & !wp) {
      self.wn = Cell(w as u32);
      if !self.draw(b) {
        r.push(usize::from(self.clone()) as u32);
      }
    }
    self.wn = wn;
    self.side *= -1;
    r
  }
  fn draw(&self, b: &Bitboards) -> bool {
    debug_assert!(self.side < 0);
    let u = !self.white_attacked_cells(b) & b.king.0[self.bk.0 as usize];
    let m = (1u64 << self.wn.0) | (1u64 << self.wr.0);
    (u & m) != 0
  }
  fn mirror_row(&mut self) {
    self.wk.mirror_row();
    self.bk.mirror_row();
    self.wn.mirror_row();
    self.wr.mirror_row();
  }
  fn mirror_col(&mut self) {
    self.wk.mirror_col();
    self.bk.mirror_col();
    self.wn.mirror_col();
    self.wr.mirror_col();
  }
  fn mirror_diag(&mut self) {
    self.wk.mirror_diag();
    self.bk.mirror_diag();
    self.wn.mirror_diag();
    self.wr.mirror_diag();
  }
  fn check(&self) -> bool {
    if self.bk.attacked_by_knight(&self.wn) {
      return true;
    }
    if self.bk.attacked_by_rook(&self.wr) {
      return true;
    }
    debug_assert!(!self.bk.attacked_by_king(&self.wk));
    false
  }
}

#[derive(Clone)]
struct Node {
  rev_adj: Vec<u32>,
  a: i8,
  refcnt: u8, //black to move
}

impl Default for Node {
  fn default() -> Self {
    Node {
      rev_adj: Vec::with_capacity(0),
      a: ILLEGAL,
      refcnt: 0,
    }
  }
}

struct TableBase(Vec<Node>);

fn white_king_start_cells() -> Vec<Cell> {
  NO_TO_CELL.iter().map(|i| Cell(*i as u32)).collect()
}

const TBL: [usize; 79] = [
  50, 10, 13, 62, 88, 232, 186, 448, 624, 883, 668, 778, 462, 689, 610, 627, 926, 768, 952, 931,
  798, 819, 737, 736, 422, 457, 528, 940, 708, 359, 186, 119, 275, 359, 115, 234, 211, 143, 324,
  252, 460, 790, 1599, 1765, 2368, 3154, 3926, 6201, 7930, 10202, 14456, 22824, 33554, 44650,
  63838, 86001, 113380, 139179, 162789, 179052, 183773, 170258, 144812, 110108, 71551, 42604,
  22753, 11085, 5734, 3583, 2382, 1440, 836, 406, 190, 107, 30, 10, 0,
];

impl TableBase {
  fn create() -> Self {
    Self(vec![Node::default(); 10 << 19])
  }
  fn new() -> Self {
    let mut tb = TableBase::create();
    tb.build();
    tb
  }
  fn add_white_to_move_pos(&mut self, mut pos: Pos, c: Option<bool>, b: &Bitboards) {
    if match c {
      Some(d) => d,
      None => pos.check(),
    } {
      return;
    }
    let m = pos.white_moves(b);
    let idx = usize::from(pos);
    debug_assert_eq!(self.0[idx].a, ILLEGAL);
    self.0[idx].a = UNKNOWN;
    let idx = idx as u32;
    for p in m {
      //p - black move
      self.0[p as usize].rev_adj.push(idx);
    }
  }
  fn add_black_to_move_pos(
    &mut self,
    mut pos: Pos,
    queue_mate: &mut Vec<u32>,
    b: &Bitboards,
  ) -> Option<bool> {
    if pos.draw(b) {
      self.0[usize::from(pos)].a = DRAW;
      None
    } else {
      let (m, c) = pos.black_moves(b);
      let idx = usize::from(pos);
      debug_assert_eq!(self.0[idx].a, ILLEGAL);
      if m.is_empty() {
        if c {
          self.0[idx].a = UNKNOWN;
          queue_mate.push(idx as u32);
        } else {
          self.0[idx].a = DRAW;
        }
      } else {
        {
          let u = &mut self.0[idx];
          u.a = UNKNOWN;
          u.refcnt = m.len() as u8;
        }
        let idx = idx as u32;
        for p in m {
          //p - white move
          self.0[p as usize].rev_adj.push(idx);
        }
      }
      Some(c)
    }
  }
  fn build(&mut self) {
    let bitboards = Bitboards::default();
    let mut queue_mate = Vec::with_capacity(TBL[0]);
    for wk in white_king_start_cells() {
      for bk in 0..=0o77 {
        let bk = Cell(bk);
        if wk == bk || wk.attacked_by_king(&bk) {
          continue;
        }
        for wn in 0..=0o77 {
          let wn = Cell(wn);
          if wk == wn || bk == wn {
            continue;
          }
          for wr in 0..=0o77 {
            let wr = Cell(wr);
            if wk == wr || bk == wr || wn == wr {
              continue;
            }
            let c = self.add_black_to_move_pos(
              Pos {
                wk,
                bk,
                wn,
                wr,
                side: -1,
              },
              &mut queue_mate,
              &bitboards,
            );
            self.add_white_to_move_pos(
              Pos {
                wk,
                bk,
                wn,
                wr,
                side: 1,
              },
              c,
              &bitboards,
            );
          } // wr
        } //wn
      } //bk
    } //wk
    for k in 1i8..127i8 {
      if queue_mate.is_empty() {
        dprintln!("k = {}", k);
        break;
      }
      let mut w = Vec::with_capacity(TBL[k as usize]);
      for i in queue_mate {
        let mut rev_adj = Vec::with_capacity(0);
        {
          let u = &mut self.0[i as usize];
          debug_assert_eq!(u.a, UNKNOWN);
          u.a = -k;
          std::mem::swap(&mut rev_adj, &mut u.rev_adj);
        }
        for p in rev_adj {
          let j = p as usize;
          if self.0[j].a == UNKNOWN {
            self.0[j].a = k;
            let mut adj = Vec::with_capacity(0);
            std::mem::swap(&mut adj, &mut self.0[j].rev_adj);
            for q in adj {
              let u = &mut self.0[q as usize].refcnt;
              *u -= 1;
              if *u == 0 {
                w.push(q);
              }
            }
          }
        }
      }
      assert_eq!(w.len(), TBL[k as usize]);
      queue_mate = w;
    }
  }
}

fn main() -> io::Result<()> {
  let stdin = io::stdin();
  let stdout = io::stdout();
  let mut o = io::BufWriter::with_capacity(2 << 20, stdout.lock());
  let tb = TableBase::new();
  for t in stdin.lock().lines().map(|s| s.unwrap()).skip(1) {
    let mut it = t.split_whitespace();
    let wk: Cell = Cell::from_str(&it.next().unwrap()).unwrap();
    let bk: Cell = Cell::from_str(&it.next().unwrap()).unwrap();
    let wn: Cell = Cell::from_str(&it.next().unwrap()).unwrap();
    let wr: Cell = Cell::from_str(&it.next().unwrap()).unwrap();
    let s = it.next().unwrap();
    let ss = vec![wk.0, bk.0, wn.0, wr.0]
      .into_iter()
      .fold(0, |acc, x| acc | (1u64 << x));
    assert_eq!(ss.count_ones(), 4);
    assert!(s == "w" || s == "b");
    let side = if s == "w" { 1 } else { -1 };
    let pos = Pos {
      wk,
      bk,
      wn,
      wr,
      side,
    };
    let res = tb.0[usize::from(pos)].a;
    assert_ne!(res, ILLEGAL);
    if res == DRAW || res == UNKNOWN {
      writeln!(o, "0")?;
    } else if side > 0 {
      writeln!(o, "#{}", res)?;
    } else {
      writeln!(o, "#{}", res.abs() - 1)?;
    }
  }
  o.flush()
}