use std::convert::From;
use std::io::prelude::*;
use std::str::FromStr;
use std::{io};
 
#[cfg(dprintln)]
#[allow(unused_macros)]
macro_rules! dprintln { ($( $args:expr ),*) => { eprintln!( $( $args ),* ); } }
 
#[cfg(not(dprintln))]
#[allow(unused_macros)]
macro_rules! dprintln {
  ($( $args:expr ),*) => {};
}
 
struct Scanner<I> {
  it: I,
}
 
impl<I: std::iter::Iterator<Item = String>> Scanner<I> {
  fn next<A: FromStr>(&mut self) -> A {
    if let Ok(x) = self.it.next().unwrap().parse::<A>() {
      x
    } else {
      panic!("parse failed")
    }
  }
}
 
#[derive(Clone, Copy, PartialEq, Debug)]
struct Cell(u32);
impl Cell {
  fn go(&self, d: &(i32, i32)) -> Option<Cell> {
    let y = self.row() as i32 + d.0;
    if y < 0 || y > 7 {
      return None;
    }
    let x = self.col() as i32 + d.1;
    if x < 0 || x > 7 {
      return None;
    }
    Some(Cell((x as u32) + ((y as u32) << 3)))
  }
  fn col(&self) -> u32 {
    self.0 & 7
  }
  fn row(&self) -> u32 {
    self.0 >> 3
  }
  fn mirror_col(&mut self) {
    let col = self.col();
    self.0 = (self.0 & 0o70) + (7 - col);
  }
  fn mirror_row(&mut self) {
    let row = self.row();
    self.0 = (self.0 & 7) + ((7 - row) << 3);
  }
  fn mirror_diag(&mut self) {
    let col = self.col();
    let row = self.row();
    self.0 = (col << 3) + row;
  }
  fn unpack(&self) -> (i32, i32) {
    (self.row() as i32, self.col() as i32)
  }
  fn delta_vector(&self, other: &Self) -> (i32, i32) {
    let (y1, x1) = self.unpack();
    let (y2, x2) = other.unpack();
    (y2 - y1, x2 - x1)
  }
  fn attacked_by_king(&self, king: &Self) -> bool {
    let (vy, vx) = self.delta_vector(king);
    vy.abs().max(vx.abs()) <= 1
  }
}

fn dd(v: (i32, i32)) -> i32 {
  v.0 * v.0 + v.1 * v.1
}
 
#[derive(Debug, PartialEq, Eq)]
struct ParseCellError(String);
 
impl FromStr for Cell {
  type Err = ParseCellError;
  fn from_str(s: &str) -> Result<Self, Self::Err> {
    dprintln!("{}", s);
    if s.len() != 2 {
      return Err(ParseCellError(String::from("illegal cell str len")));
    }
    let mut it = s.chars();
    let c = it.next().unwrap();
    if c < 'a' || c > 'h' {
      return Err(ParseCellError(String::from("illegal col character")));
    }
    let col = c as u8 - b'a';
    let c = it.next().unwrap();
    if c < '1' || c > '8' {
      return Err(ParseCellError(String::from("illegal row character")));
    }
    let row = c as u8 - b'1';
    Ok(Cell(8 * row as u32 + col as u32))
  }
}

const KNIGHT_DIRECTIONS: [(i32, i32); 8] = [
  (3, 1), (1, 3),
  (-3, 1), (-1, 3),
  (3, -1), (1, -3),
  (-3, -1), (-1, -3),
];
 
const DIRECTIONS: [(i32, i32); 8] = [
  (-1, 0),
  (0, -1),
  (0, 1),
  (1, 0),
  (-1, -1),
  (-1, 1),
  (1, -1),
  (1, 1),
];
const ILLEGAL: i8 = 127;
const DRAW: i8 = 126;
const UNKNOWN: i8 = 125;
 
#[derive(Clone, Debug, PartialEq)]
struct Pos {
  wk: Cell,
  bk: Cell,
  wn: Cell, //верблюд
  wr: Cell, //визирь
  side: i8, /* 1 - white, -1 - black */
}
 
impl From<u32> for Pos {
  fn from(x: u32) -> Pos {
    let wkx = x & 3;
    let wky = (x >> 2) & 3;
    let wk = Cell(wky * 8 + wkx);
    let bk = Cell((x >> 4) & 63);
    let wn = Cell((x >> 10) & 63);
    let wr = Cell((x >> 16) & 63);
    let side = if (x & (1 << 22)) != 0 { -1 } else { 1 };
    Pos {
      wk,
      bk,
      wn,
      wr,
      side,
    }
  }
}

impl From<Pos> for usize {
  fn from(p: Pos) -> usize {
    let mut p = p;
    if p.wk.col() >= 4 {
      p.mirror_col();
    }
    if p.wk.row() >= 4 {
      p.mirror_row();
    }
    if p.wk.row() > p.wk.col() {
      p.mirror_diag();
    }
    let r = if p.side < 0 { 1 << 22 } else { 0 }
      + (p.wr.0 << 16)
      + (p.wn.0 << 10)
      + (p.bk.0 << 4)
      + (p.wk.row() << 2)
      + (p.wk.col());
    r as _
  }
}
 
impl Pos {
  fn black_moves(&mut self) -> Vec<u32> {
    debug_assert_eq!(self.side, -1);
    let mut r = Vec::with_capacity(8);
    let bk = self.bk;
    self.side *= -1;
    for d in &DIRECTIONS {
      if let Some(w) = bk.go(d) {
        if w.attacked_by_king(&self.wk) {
          continue;
        }
        self.bk = w;
        if !self.check() {
          r.push(usize::from(self.clone()) as u32);
        }
      }
    }
    self.bk = bk;
    self.side *= -1;
    r
  }
  fn white_moves(&mut self) -> Vec<u32> {
    debug_assert_eq!(self.side, 1);
    self.side *= -1;
    let mut r = Vec::new();
    //king moves
    let wk = self.wk;
    for d in &DIRECTIONS {
      if let Some(w) = wk.go(d) {
        if w.attacked_by_king(&self.bk) || w == self.wn || w == self.wr {
          continue;
        }
        self.wk = w;
        r.push(usize::from(self.clone()) as u32);
      }
    }
    self.wk = wk;
    
    let wr = self.wr;
    for d in DIRECTIONS.iter().take(4) {
      if let Some(w) = wr.go(d) {
        if w == self.wk || w == self.wn { continue; }
        self.wr = w;
        r.push(usize::from(self.clone()) as u32);
      }
    }
    self.wr = wr;

    let wn = self.wn;
    for d in &KNIGHT_DIRECTIONS {
      if let Some(w) = wn.go(d) {
        if w == self.wk || w == self.wr { continue; }
        self.wn = w;
        r.push(usize::from(self.clone()) as u32);
      }
    }
    self.wn = wn;
    self.side *= -1;
    r
  }
  fn draw(&self) -> bool {
    debug_assert!(self.side < 0);
    (self.bk.attacked_by_king(&self.wn) && !self.wk.attacked_by_king(&self.wn) && dd(self.wr.delta_vector(&self.wn)) != 1) ||
    (self.bk.attacked_by_king(&self.wr) && !self.wk.attacked_by_king(&self.wr) && dd(self.wn.delta_vector(&self.wr)) != 10)
  }
  fn mirror_row(&mut self) {
    self.wk.mirror_row();
    self.bk.mirror_row();
    self.wn.mirror_row();
    self.wr.mirror_row();
  }
  fn mirror_col(&mut self) {
    self.wk.mirror_col();
    self.bk.mirror_col();
    self.wn.mirror_col();
    self.wr.mirror_col();
  }
  fn mirror_diag(&mut self) {
    self.wk.mirror_diag();
    self.bk.mirror_diag();
    self.wn.mirror_diag();
    self.wr.mirror_diag();
  }
  fn check(&self) -> bool {
    if dd(self.bk.delta_vector(&self.wn)) == 10 {
      return true;
    }
    if dd(self.bk.delta_vector(&self.wr)) == 1 {
      return true;
    }
    self.bk.attacked_by_king(&self.wk)
  }
}
 
#[derive(Clone)]
struct Node {
  rev_adj: Vec<u32>,
  a: i8,
  refcnt: u8,
}
 
impl Default for Node {
  fn default() -> Self {
    Node {
      rev_adj: Vec::with_capacity(0),
      a: ILLEGAL,
      refcnt: 0,
    }
  }
}
 
struct TableBase(Vec<Node>);

fn white_king_start_cells() -> Vec<Cell> {
  let mut a = Vec::new();
  for x in 0 .. 4 {
    for y in 0 ..= x {
      a.push(Cell(y * 8 + x));
    }
  }
  a
}

impl TableBase {
  fn create() -> Self {
    Self(vec![Node::default(); 1 << 23])
  }
  fn new() -> Self {
    let mut tb = TableBase::create();
    tb.build();
    tb
  }
  fn add_white_to_move_pos(&mut self, mut pos: Pos, c: Option<bool>) {
    if match c {
      Some(d) => d,
      None => pos.check(),
    } {
      return;
    }
    let m = pos.white_moves();
    let idx = usize::from(pos);
    debug_assert_eq!(self.0[idx].a, ILLEGAL);
    self.0[idx].a = UNKNOWN;
    for p in m {
      self.0[p as usize].rev_adj.push(idx as u32);
    }
  }
  fn add_black_to_move_pos(&mut self, mut pos: Pos, queue_mate: &mut Vec<u32>) -> Option<bool> {
    if pos.draw() {
      self.0[usize::from(pos)].a = DRAW;
      None
    } else {
      let m = pos.black_moves();
      let c = pos.check();
      let idx = usize::from(pos);
      debug_assert_eq!(self.0[idx].a, ILLEGAL);
      if c && m.is_empty() {
        self.0[idx].a = UNKNOWN;
        queue_mate.push(idx as u32);
      } else {
        if m.is_empty() {
          self.0[idx].a = DRAW;
        } else {
          {
            let u = &mut self.0[idx];
            u.a = UNKNOWN;
            u.refcnt = m.len() as u8;
          }
          for p in m {
            self.0[p as usize].rev_adj.push(idx as u32);
          }
        }
      }
      Some(c)
    }
  }
  fn build(&mut self) {
    let mut queue_mate = Vec::new();
    for wk in white_king_start_cells() {
      for bk in 0..=0o77 {
        let bk = Cell(bk);
        if wk == bk || wk.attacked_by_king(&bk) { continue; }
        for wn in 0 ..= 0o77 {
          let wn = Cell(wn);
          if wk == wn || bk == wn { continue; }
          for wr in 0 ..= 0o77 {
            let wr = Cell(wr);
            if wk == wr || bk == wr || wn == wr { continue; }
            let c = self.add_black_to_move_pos(
              Pos {
                wk,
                bk,
                wn,
                wr,
                side: -1,
              },
              &mut queue_mate,
            );
            self.add_white_to_move_pos(
              Pos {
                wk,
                bk,
                wn,
                wr,
                side: 1,
              },
              c,
            );
          } // wr
        } //wn
      } //bk
    } //wk
    for k in 1i8..127i8 {
      //eprintln!("k = {}", k);
      if queue_mate.is_empty() {
        dprintln!("k = {}", k);
        break;
      }
      let mut w = Vec::new();
      for i in queue_mate {
        let mut rev_adj = Vec::with_capacity(0);
        {
          let u = &mut self.0[i as usize];
          debug_assert_eq!(u.a, UNKNOWN);
          u.a = -k;
          std::mem::swap(&mut rev_adj, &mut u.rev_adj);
        }
        for p in rev_adj {
          let j = p as usize;
          if self.0[j].a == UNKNOWN {
            self.0[j].a = k;
            let mut adj = Vec::with_capacity(0);
            std::mem::swap(&mut adj, &mut self.0[j].rev_adj);
            for q in adj {
              let u = &mut self.0[q as usize];
              let t = u.refcnt;
              debug_assert!(t > 0);
              if t == 1 {
                w.push(q);
              }
              u.refcnt -= 1;
            }
            //dprintln!("{}: {}, idx = {}", k, Pos::from(p), j);
          }
        }
      }
      queue_mate = w;
    }
  }
}
 
fn main() -> io::Result<()> {
  let stdin = io::stdin();
  let it = stdin.lock().lines().flat_map(|s| {
    s.unwrap()
      .split_whitespace()
      .map(|s| s.to_owned())
      .collect::<Vec<_>>()
  });
  let mut r = Scanner { it };
  let stdout = io::stdout();
  let mut o = io::BufWriter::with_capacity(1 << 20, stdout.lock());
  let tb = TableBase::new();
  for _ in 0..r.next::<usize>() {
    let wk = r.next::<Cell>();
    let bk = r.next::<Cell>();
    let wn = r.next::<Cell>();
    let wr = r.next::<Cell>();
    let s = r.it.next().unwrap();
    let side = if s == "w" { 1 } else { -1 };
    let pos = Pos {
      wk,
      bk,
      wn,
      wr,
      side,
    };
    let res = tb.0[usize::from(pos)].a;
    assert_ne!(res, ILLEGAL);
    if res == DRAW || res == UNKNOWN {
      writeln!(o, "0")?;
    } else if side > 0 {
      writeln!(o, "#{}", res)?;
    } else {
      writeln!(o, "#{}", res.abs() - 1)?;
    }
  }
  o.flush()
}

#[cfg(test)]
fn make_pos(s: &str) -> Pos {
  let it = s.split_whitespace().map(|t| t.to_owned());
  let mut r = Scanner{it};
  let wk = r.next::<Cell>();
  let bk = r.next::<Cell>();
  let wn = r.next::<Cell>();
  let wr = r.next::<Cell>();
  let s = r.it.next().unwrap();
  let side = if s == "w" { 1 } else { -1 };
  Pos {
    wk,
    bk,
    wn,
    wr,
    side,
  }
}

#[test]
fn test_check() {
  let pos = make_pos("f3 f1 f4 f2 b");
  assert!(pos.check());
  let k = usize::from(pos.clone());
  let pos2 = Pos::from(k as u32);
  assert!(pos2.check());
  let pos = make_pos("f3 f1 g4 g2 b");
  assert!(pos.check());
}

#[test]
fn test_pack() {
  let pos = make_pos("b1 f1 f4 f2 b");
  let k = usize::from(pos.clone());
  let pos2 = Pos::from(k as u32);
  assert_eq!(pos, pos2);
  let pos = make_pos("b1 h8 f4 f2 w");
  let k = usize::from(pos.clone());
  let pos2 = Pos::from(k as u32);
  assert_eq!(pos, pos2);
}

#[test]
fn test_black_moves() {
  let mut pos = make_pos("f3 f1 f4 f2 b");
  assert!(pos.black_moves().is_empty());
  assert!(!pos.draw());
  let mut pos = make_pos("c3 c1 c4 c2 b");
  assert!(pos.black_moves().is_empty());
  assert!(!pos.draw());
}