#include <iostream>
#include <algorithm>
#include <iterator>
#include <vector>
#include <ranges>
#include <sstream>
#include <iomanip>

struct Point {
    int32_t x, y;
    bool operator!=(const Point other) const {
        return x != other.x || y != other.y;
    }
};

struct Left {
    bool operator()(const Point a, const Point b) const {
        return a.x < b.x;
    }
};

struct Right {
    bool operator()(const Point a, const Point b) const {
        return a.x > b.x;
    }
};

struct FarFromOx {
    bool operator()(const Point a, const Point b) const {
        return abs(a.y) > abs(b.y);
    }
};

struct Up {
    bool operator()(const Point a, const Point b) const {
        return a.y > b.y;
    }
};

struct Down {
    bool operator()(const Point a, const Point b) const {
        return a.y < b.y;
    }
};

std::istream& operator>>(std::istream& in, Point& point) {
    return in >> point.x >> point.y;
}

std::string generate_metapost_picture(const std::vector<Point>& points, const Point leftmost, const Point rightmost, const Point vertical) {
    std::stringstream pic;
    pic << std::format("beginfig(0);\n");
    pic << std::format("\tu = 1cm;\n");
    for (uint32_t i = 0; i < points.size(); ++i)
        pic << std::format("\tz{} = ({}u, {}u);\n", i, points[i].x, points[i].y);
    int32_t min_x = std::min(0, std::ranges::min(points, Left()).x);
    int32_t max_x = std::max(0, std::ranges::min(points, Right()).x);
    int32_t min_y = std::min(0, std::ranges::min(points, Down()).y);
    int32_t max_y = std::max(0, std::ranges::min(points, Up()).y);
    pic << std::format("\tnumeric minx, maxx, miny, maxy, offsetx, offsety, n;\n");
    pic << std::format("\toffsetx = {}u;\n", .5);
    pic << std::format("\toffsety = {}u;\n", .5);
    pic << std::format("\tminx = {};\n", min_x);
    pic << std::format("\tmaxx = {};\n", max_x);
    pic << std::format("\tminy = {};\n", min_y);
    pic << std::format("\tmaxy = {};\n", max_y);
    pic << std::format("\tn = {};\n", points.size());
    if (leftmost != rightmost && vertical.y) {
        pic << std::format("\tdraw ({}u, {}u) -- ({}u, {}u) -- ({}u, {}u);\n", leftmost.x, leftmost.y, vertical.x, vertical.y, rightmost.x, rightmost.y);
        pic << std::format("\tfill ({}u, {}u) -- ({}u, {}u) -- ({}u, {}u) -- cycle withcolor .6white;\n", leftmost.x, leftmost.y, vertical.x, vertical.y, rightmost.x, rightmost.y);
    }
    pic << std::format("\tfor i := minx upto maxx:\n");
    pic << std::format("\t\tif i <> 0:\n");
    pic << std::format("\t\t\tdraw (i * u, miny * u - offsety) -- (i * u, maxy * u + offsety) dashed evenly;\n");
    pic << std::format("\t\tfi;\n");
    pic << std::format("\tendfor;\n");
    pic << std::format("\tfor i := miny upto maxy:\n");
    pic << std::format("\t\tif i <> 0:\n");
    pic << std::format("\t\t\tdraw (minx * u - offsetx, i * u) -- (maxx * u + offsetx, i * u) dashed evenly;\n");
    pic << std::format("\t\tfi;\n");
    pic << std::format("\tendfor;\n");
    pic << std::format("\tdrawarrow (0, miny * u - offsety) -- (0, maxy * u + offsety);\n");
    pic << std::format("\tdrawarrow (minx * u - offsetx, 0) -- (maxx * u + offsetx, 0);\n");
    pic << std::format("\tlabel.llft(btex $O$ etex, (0, 0));\n");
    pic << std::format("\tlabel.llft(btex $x$ etex, (maxx * u + offsetx, 0));\n");
    pic << std::format("\tlabel.llft(btex $y$ etex, (0, maxy * u + offsety));\n");
    pic << std::format("\tpickup pencircle scaled 4pt;\n");
    pic << std::format("\tfor i := 0 upto n - 1:\n");
    pic << std::format("\t\tdrawdot z[i];\n");
    pic << std::format("\tendfor;\n");
    pic << std::format("endfig;\n");
    pic << std::format("bye\n");
    return pic.str();
}

int main() {
    uint32_t n;
    std::cin >> n;
    std::vector<Point> points(n);
    std::copy_n(std::istream_iterator<Point>(std::cin), n, points.begin());
    Point leftmost = std::ranges::min(points | std::ranges::views::filter([](const Point p){return p.y == 0;}), Left()), 
          rightmost = std::ranges::min(points | std::ranges::views::filter([](const Point p){return p.y == 0;}), Right()), 
          vertical = std::ranges::min(points, FarFromOx());
    std::cout << std::fixed << std::setprecision(1) << (rightmost.x - leftmost.x) * abs(vertical.y) * (long double).5 << '\n';
    std::cerr << generate_metapost_picture(points, leftmost, rightmost, vertical);
}
