#include <iostream>
#include <algorithm>
#include <iterator>
#include <vector>
#include <ranges>
#include <sstream>

struct Point {
    int32_t x, y;
    bool operator!=(const Point other) const {
        return x != other.x || y != other.y;
    }
};

struct Left {
    bool operator()(const Point a, const Point b) {
        return a.x < b.x;
    }
};

struct Right {
    bool operator()(const Point a, const Point b) {
        return a.x > b.x;
    }
};

struct FarFromOx {
    bool operator()(const Point a, const Point b) {
        return abs(a.y) > abs(b.y);
    }
};

struct Up {
    bool operator()(const Point a, const Point b) {
        return a.y > b.y;
    }
};

struct Down {
    bool operator()(const Point a, const Point b) {
        return a.y < b.y;
    }
};

std::istream& operator>>(std::istream& in, Point& point) {
    return in >> point.x >> point.y;
}

char to_hex_digit(const uint8_t n) {
    return n <= 9 ? '0' + n : 'a' + (n - 10);
}

std::string to_hex(const uint64_t n) {
    std::string hex(16, ' ');
    for (int i = 15; i >= 0; --i)
        hex[15 - i] = to_hex_digit(n >> i * 4 & 15);
    return hex;
}

std::string get_hash(const std::vector<Point>& points) {
    std::stringstream list_of_points;
    for (const Point point : points)
        list_of_points << point.x << ' ' << point.y << '\n';
    return to_hex(std::hash<std::string>()(list_of_points.str()));
}

std::string generate_metapost_picture(const std::vector<Point>& points, const Point leftmost, const Point rightmost, const Point vertical) {
    std::stringstream pic;
    std::string name = get_hash(points);
    pic << std::format("beginfig({});\n", name);
    pic << "\tu = 1cm;\n";
    for (uint32_t i = 0; i < points.size(); ++i)
        pic << std::format("\tz{} = ({}u, {}u);\n", i, points[i].x, points[i].y);
    int32_t min_x = std::min(0, std::ranges::min(points, Left()).x);
    int32_t max_x = std::max(0, std::ranges::min(points, Right()).x);
    int32_t min_y = std::min(0, std::ranges::min(points, Down()).y);
    int32_t max_y = std::max(0, std::ranges::min(points, Up()).y);
    pic << std::format("\tnumeric minx, maxx, miny, maxy, offsetx, offsety;\n");
    pic << std::format("\toffsetx = {}u;\n", .5);
    pic << std::format("\toffsety = {}u;\n", .5);
    pic << std::format("\tminx = {};\n", min_x);
    pic << std::format("\tmaxx = {};\n", max_x);
    pic << std::format("\tminy = {};\n", min_x);
    pic << std::format("\tmaxy = {};\n", max_y);
    if (leftmost != rightmost && vertical.y) {
        pic << std::format("\tdraw ({}u, {}u) -- ({}u, {}u) -- ({}u, {}u);\n", leftmost.x, leftmost.y, vertical.x, vertical.y, rightmost.x, rightmost.y);
        pic << std::format("\tfill ({}u, {}u) -- ({}u, {}u) -- ({}u, {}u) -- cycle withcolor .6white;\n", leftmost.x, leftmost.y, vertical.x, vertical.y, rightmost.x, rightmost.y);
    }
    pic << std::format("\tfor i := minx upto maxx:\n");
    pic << std::format("\t\tif i <> 0:\n");
    pic << std::format("\t\t\tdraw (i * u, miny * u - offsety) -- (i * u, maxy * u + offsety) dashed evenly;\n");
    pic << std::format("\t\tfi;\n");
    pic << std::format("\tendfor;\n");
    pic << std::format("\tfor i := miny upto maxy:\n");
    pic << std::format("\t\tif i <> 0:\n");
    pic << std::format("\t\t\tdraw (minx * u - offsetx, i * u) -- (maxx * u + offsetx, i * u) dashed evenly;\n");
    pic << std::format("\t\tfi;\n");
    pic << std::format("\tendfor;\n");
    pic << std::format("endfig;\n");
    pic << std::format("bye\n");
    return pic.str();
}

int main() {
    uint32_t n;
    std::cin >> n;
    std::vector<Point> points(n);
    std::copy_n(std::istream_iterator<Point>(std::cin), n, points.begin());
    Point leftmost = std::ranges::min(points | std::ranges::views::filter([](const Point p){return p.y == 0;}), Left()), 
          rightmost = std::ranges::min(points | std::ranges::views::filter([](const Point p){return p.y == 0;}), Right()), 
          vertical = std::ranges::min(points, FarFromOx());
    std::cout << (rightmost.x - leftmost.x) * abs(vertical.y) * .5 << '\n';
    std::cerr << generate_metapost_picture(points, leftmost, rightmost, vertical);
}
