#include "testlib.h"
#include <ranges>

const uint64_t MAX_M = 1e9;

uint32_t gcd(uint32_t a, uint32_t b) {
    while (b) {
        a %= b;
        std::swap(a, b);
    }
    return a;
}

struct Residue {
    uint32_t modulus;
    int32_t representative;
    
    Residue operator&(const Residue& other) && {
        modulus = gcd(modulus, other.modulus);
        modulus = gcd(modulus, abs(representative - other.representative));
        return *this;
    }
    
    Residue& operator&=(const Residue other) {
        return *this = std::move(*this) & other;
    }
    
    Residue operator&(const Residue other) const {
        Residue ans = *this;
        return ans &= other;
    }
};

std::optional<Residue>& operator&=(std::optional<Residue>& a, Residue b) {
    if (!a)
        return a = b;
    *a &= b;
    return a;
}

struct ResidueSegmentTree {
    std::vector<Residue> v;
    
    uint32_t n() const {
        return v.size() >> 1;
    }
    
    ResidueSegmentTree(const std::vector<int32_t>& a) : v(a.size() << 1) {
        std::ranges::copy(a | std::ranges::views::transform([](int32_t b) {
            return Residue{.modulus = 0, .representative = b};
        }), v.begin() + n());
        for (uint32_t i = n() - 1; i > 0; --i)
            v[i] = v[2 * i] & v[2 * i + 1];
    }
    
    std::optional<Residue> segment_conjunction(uint32_t l, uint32_t r) const {
        l += n(); r += n();
        std::optional<Residue> ans;
        while (l < r) {
            if (l & 1) ans &= v[l++];
            if (r & 1) ans &= v[--r];
            l >>= 1; r >>= 1;
        }
        return ans;
    }
};

std::string english_plural(const uint32_t n) {
    return n == 1 ? "" : "s";
}

std::optional<uint32_t> read_and_check_ans(const ResidueSegmentTree& residue_segment_tree, const uint32_t s, const uint32_t f, InStream& in) {
    uint32_t current_city = s;
    uint32_t taxis = 0;
    while (!in.seekEof()) {
        ++taxis;
        in.ensuref(taxis <= residue_segment_tree.n() * 2, "the travel took exceptionally many taxis");
        std::string Kaban;
        if (taxis >= 2)
            Kaban = in.readToken("[Kk][Aa][Bb][Aa][Nn]-[1-9][0-9]{0,9}", std::format("Kaban[{}]", taxis));
        else {
            Kaban = in.readToken("([Kk][Aa][Bb][Aa][Nn]-[1-9][0-9]{0,9})|([Ii][Mm][Pp][Oo][Ss][Ss][Ii][Bb][Ll][Ee])", std::format("Kaban[{}] or Impossible", taxis));
            if (Kaban[0] == 'I' || Kaban[0] == 'i')
                return {};
        }
        uint64_t mi = atoll(Kaban.c_str() + 6);
        in.ensuref(2 <= mi && mi <= MAX_M, "%s", std::format("m[{}] == {}, but it should be between {} and {}",
                                                             taxis, mi, 2, MAX_M).c_str());
        uint32_t si = in.readInt(1, residue_segment_tree.n(), std::format("s[{}]", taxis));
        uint32_t fi = in.readInt(1, residue_segment_tree.n(), std::format("f[{}]", taxis));
        std::optional<Residue> residue = residue_segment_tree.segment_conjunction(std::min(si, fi) - 1, std::max(si, fi));
        if (!residue.has_value())
            in.quitf(_fail, "%s", std::format("the {}{} taxi segment {}--{} chances to have an empty residue",
                                           taxis, englishEnding(taxis), si, fi).c_str());
        in.ensuref(residue->modulus % mi == 0 || residue->modulus != 1, "%s", std::format("the {}{} taxi segment {}--{} is not Kaban-{}'able",
                                                                 taxis, englishEnding(taxis), si, fi, mi).c_str());
        in.ensuref(residue->modulus % mi == 0 || residue->modulus == 1, "%s", std::format("the {}{} taxi segment {}--{} is not Kaban-{}'able but is Kaban-{}'able",
                                                                 taxis, englishEnding(taxis), si, fi, mi, std::max<uint32_t>(2, residue->modulus)).c_str());
        in.ensuref(current_city == si, "%s", std::format("the {}{} taxi segment starts at {}, but should at {}",
                                                         taxis, englishEnding(taxis), si, current_city).c_str());
        current_city = fi;
    }
    in.ensuref(current_city == f, "%s", std::format("the {}{} taxi segment ends at {}, but should at {}",
                                                     taxis, englishEnding(taxis), current_city, f).c_str());
    return taxis;
}

int main(int argc, char* argv[]) {
    registerTestlibCmd(argc, argv);
    uint32_t n = inf.readInt(std::numeric_limits<int32_t>::min(), std::numeric_limits<int32_t>::max(), "n");
    ResidueSegmentTree residue_segment_tree(
        std::ranges::views::iota(static_cast<uint32_t>(1), n + 1) | std::ranges::views::transform([](int32_t i) {
            return inf.readInt(std::numeric_limits<int32_t>::min(), std::numeric_limits<int32_t>::max(), std::format("a[{}]", i));
        }) | std::ranges::to<std::vector<int32_t>>()
    );
    uint32_t s = inf.readInt(std::numeric_limits<int32_t>::min(), std::numeric_limits<int32_t>::max(), "s");
    uint32_t f = inf.readInt(std::numeric_limits<int32_t>::min(), std::numeric_limits<int32_t>::max(), "f");
    std::optional<uint32_t> jury = read_and_check_ans(residue_segment_tree, s, f, ans);
    std::optional<uint32_t> part = read_and_check_ans(residue_segment_tree, s, f, ouf);
    if (jury.has_value() != part.has_value())
        if (jury.has_value())
            quitf(_wa, "%s", std::format("jury's route of {} taxi{} is better than participant's impossibility",
                                         *jury, english_plural(*jury)).c_str());
        else
            quitf(_fail, "%s", std::format("jury's impossibility is worse than participant's route of {} taxi{}",
                                           *part, english_plural(*part)).c_str());
    else if (jury != part)
        if (*jury < *part)
            quitf(_wa, "%s", std::format("jury's route of {} taxi{} is better than participant's route of {} taxi{}",
                                         *jury, english_plural(*jury), *part, english_plural(*part)).c_str());
        else
            quitf(_fail, "%s", std::format("jury's route of {} taxi{} is worse than participant's route of {} taxi{}",
                                           *jury, english_plural(*jury), *part, english_plural(*part)).c_str());
    else if (jury.has_value())
        quitf(_ok, "%s", std::format("{} taxi{}", *jury, english_plural(*jury)).c_str());
    else
        quitf(_ok, "%s", std::format("Impossible").c_str());
}
